<?php

// don't load directly
if ( ! defined( 'ABSPATH' ) ) {
	die();
}

GFForms::include_feed_addon_framework();

/**
 * Gravity Forms MailCamp Add-On.
 *
 * @since     0.1
 * @package   GravityForms
 * @author    Imagro
 * @copyright Copyright (c) 2022, Imagro
 */
class GFMailCamp extends GFFeedAddOn {
	private static $_instance 	= null;
	protected $_languageslug 	= 'gravityformsmailcamp';
	
	protected $_version 		= '0.2';
	protected $_min_gravityforms_version = '1.9';
	protected $_slug 			= 'mailcamp-imagro';
	protected $_path 			= 'mailcamp-imagro/mailcamp-imagro.php';
	protected $_full_path 		= __FILE__;
	protected $_title 			= 'Gravity Forms Mailcamp Add-On';
	protected $_short_title 	= 'Mailcamp';
	
	/**
	 * Contains an instance of the Mailcamp API library, if available.
	 * @since  0.1
	 *
	 * @var    GF_MailCamp_API $api If available, contains an instance of the Mailcamp API library.
	 */
	public $api = null;
	
	
	/**
	 * Get an instance of this class.
	 * @since  0.1
	 *
	 * @return GFMailCamp
	 */
	public static function get_instance() {

		if ( null === self::$_instance ) {
			self::$_instance = new self;
		}

		return self::$_instance;

	}
	/**
	 * Autoload the required libraries.
	 *
	 * @since  4.0
	 * @access public
	 *
	 * @uses GFAddOn::is_gravityforms_supported()
	 */
	public function pre_init() {

		parent::pre_init();

		if ( $this->is_gravityforms_supported() ) {

			// Load the Mailcamp API library.
			if ( ! class_exists( 'GF_MailCamp_API' ) ) {
				require_once( __DIR__.'/class-gf-mailcamp-api.php' );
			}

		}

	}
	
	/**
	 * Return the plugin's icon for the plugin/form settings menu.
	 * @since 0.2
	 *
	 * @return string
	 */
	public function get_menu_icon() {

		return $this->is_gravityforms_supported( '2.5-beta-4' ) ? 'gform-icon-mailcamp' : 'dashicons-admin-generic';

	}
	
	/**
	 * Configures which columns should be displayed on the feed list page.
	 * @since  0.1
	 *
	 * @return array
	 */
	public function feed_list_columns() {
		return array(
			'feedName'            	=> esc_html__( 'Name', $this->_languageslug ),
			'mailcamp_list_name'	=> esc_html__( 'Mailcamp Mailinglist', $this->_languageslug ),
		);
	}
	
	/**
	 * Returns the value to be displayed in the Mailcamp MailingList/Audience column.
	 * @since  1.0
	 *
	 * @param array $feed The feed being included in the feed list/audience.
	 *
	 * @return string
	 */
	public function get_column_value_mailcamp_list_name( $feed ) {

		// If unable to initialize API, return the list/audience ID.
		if ( ! $this->initialize_api() ) {
			return rgars( $feed, 'meta/mailcampList' );
		}
		

		try {
			// Get list/audience.
			$list = $this->api->get_list( rgars( $feed, 'meta/mailcampList' ) );

			// Return list/audience name.
			return rgar( $list, 'name' );

		} catch ( Exception $e ) {

			// Log error.
			$this->log_error( __METHOD__ . '(): Unable to get Mailcamp Mailinglist; ' . $e->getMessage() );

			// Return list/audience ID.
			return rgars( $feed, 'meta/mailcampList' );

		}

	}
	

	// # FEED PROCESSING -----------------------------------------------------------------------------------------------

	/**
	 * Process the feed, subscribe the user to the list/audience.
	 * @since  0.2
	 *
	 * @param array $feed  The feed object to be processed.
	 * @param array $entry The entry object currently being processed.
	 * @param array $form  The form object currently being processed.
	 *
	 * @return array
	 */
	public function process_feed( $feed, $entry, $form ) {
		
		// Log that we are processing feed.
		$this->log_debug( __METHOD__ . '(): Processing feed.' );

		// If unable to initialize API, log error and return.
		if ( ! $this->initialize_api() ) {
			$this->add_feed_error( esc_html__( 'Unable to process feed because API could not be initialized.', $this->_languageslug ), $feed, $entry, $form );
			return $entry;
		}

		// Set current merge variable name.
		$this->merge_var_name = 'EMAIL';

		// Get field map values.
		$field_map = $this->get_field_map_fields( $feed, 'mappedFields' );

		// Get mapped email address.
		$email = $this->get_field_value( $form, $entry, $field_map['EMAIL'] );

		// If email address is invalid, log error and return.
		if ( GFCommon::is_invalid_or_empty_email( $email ) ) {
			$this->add_feed_error( esc_html__( 'A valid Email address must be provided.', $this->_languageslug ), $feed, $entry, $form );
			return $entry;
		}
		
		// Initialize array to store merge vars.
		$merge_vars = array();
		$merge_vars['email'] = $email;

		// Loop through field map.
		foreach ( $field_map as $name => $field_id ):

			// If no field is mapped, skip it.
			if ( rgblank( $field_id ) ) {
				continue;
			}

			// If this is the email field, skip it.
			if ( strtoupper( $name ) === 'EMAIL' ) {
				continue;
			}

			// Set merge var name to current field map name.
			$this->merge_var_name = $name;

			// Get field object.
			$field = GFFormsModel::get_field( $form, $field_id );
		
			$field_value = $this->get_field_value( $form, $entry, $field_id );
			$merge_field = $this->get_list_merge_field( $feed['meta']['mailcampList'], $name );

			// Format date field.
			if ( ! empty( $field_value ) && ! empty( $merge_field ) && in_array( $merge_field['type'], array( 'date', 'birthday' ) ) ) {

				// Get date format.
				$date_format = $merge_field['options']['date_format'];

				// Convert field value to timestamp.
				$field_value_timestamp = strtotime( $field_value );

				// Format date.
				switch ( $date_format ) {
					case 'DD/MM':
					case 'MM/DD':
						$field_value = date( 'm/d', $field_value_timestamp );
						break;

					case 'DD/MM/YYYY':
					case 'MM/DD/YYYY':
						$field_value = date( 'm/d/Y', $field_value_timestamp );
						break;

				}

			}

			$merge_vars[ $name ] = $field_value;
		endforeach;
		
		// Other settings 
		$params = [
			'contact__confirmStatus' 	=> $feed['meta']['contact__confirmStatus'],
			'contact__autoresponders' 	=> $feed['meta']['contact__autoresponders'],
		];
		
		if($this->api->checkSubscriber_exists_in_Mailinglist($feed['meta']['mailcampList'], $email) !== false):
			// Update subscriber
			$this->api->updateSubscriber($email, $feed['meta']['mailcampList'], $merge_vars, $params);
		else:
			// Add subscriber
			$this->api->addSubscriber($feed['meta']['mailcampList'], $merge_vars, $params);
		endif;
		return;
	}
	

	/**
	 * Returns the value of the selected field.
	 * @since  0.2
	 *
	 * @param array  $form     The form object currently being processed.
	 * @param array  $entry    The entry object currently being processed.
	 * @param string $field_id The ID of the field being processed.
	 *
	 * @uses GFAddOn::get_full_name()
	 * @uses GF_Field::get_value_export()
	 * @uses GFFormsModel::get_field()
	 * @uses GFFormsModel::get_input_type()
	 * @uses GFMailCamp::get_full_address()
	 * @uses GFMailCamp::maybe_override_field_value()
	 *
	 * @return array|string
	 */
	public function get_field_value( $form, $entry, $field_id ) {

		// Set initial field value.
		$field_value = '';

		// Set field value based on field ID.
		switch ( strtolower( $field_id ) ) {

			// Form title.
			case 'form_title':
				$field_value = rgar( $form, 'title' );
				break;

			// Entry creation date.
			case 'date_created':

				// Get entry creation date from entry.
				$date_created = rgar( $entry, strtolower( $field_id ) );

				// If date is not populated, get current date.
				$field_value = empty( $date_created ) ? gmdate( 'Y-m-d H:i:s' ) : $date_created;
				break;

			// Entry IP and source URL.
			case 'ip':
			case 'source_url':
				$field_value = rgar( $entry, strtolower( $field_id ) );
				break;

			default:

				// Get field object.
				$field = GFFormsModel::get_field( $form, $field_id );

				if ( is_object( $field ) ) {

					// Check if field ID is integer to ensure field does not have child inputs.
					$is_integer = $field_id == intval( $field_id );

					// Get field input type.
					$input_type = GFFormsModel::get_input_type( $field );

					/*
					if ( $is_integer && 'address' === $input_type ) {

						// Get full address for field value.
						$field_value = $this->get_full_address( $entry, $field_id );

					} else if ( $is_integer && 'name' === $input_type ) {

						// Get full name for field value.
						$field_value = $this->get_full_name( $entry, $field_id );

					} else */
					if ( $is_integer && 'checkbox' === $input_type ) {

						// Initialize selected options array.
						$selected = array();

						// Loop through checkbox inputs.
						foreach ( $field->inputs as $input ) {
							$index = (string) $input['id'];
							if ( ! rgempty( $index, $entry ) ) {
								$selected[] = $this->maybe_override_field_value( rgar( $entry, $index ), $form, $entry, $index );
							}
						}

						// Convert selected options array to comma separated string.
						$field_value = implode( ', ', $selected );

					} else if ( 'phone' === $input_type && $field->phoneFormat == 'standard' ) {

						// Get field value.
						$field_value = rgar( $entry, $field_id );

						// Reformat standard format phone to match Mailchimp format.
						// Format: NPA-NXX-LINE (404-555-1212) when US/CAN.
						if ( ! empty( $field_value ) && preg_match( '/^\D?(\d{3})\D?\D?(\d{3})\D?(\d{4})$/', $field_value, $matches ) ) {
							$field_value = sprintf( '%s-%s-%s', $matches[1], $matches[2], $matches[3] );
						}

					} else {

						// Use export value if method exists for field.
						if ( is_callable( array( 'GF_Field', 'get_value_export' ) ) ) {
							$field_value = $field->get_value_export( $entry, $field_id );
						} else {
							$field_value = rgar( $entry, $field_id );
						}

					}

				} else {

					// Get field value from entry.
					$field_value = rgar( $entry, $field_id );

				}

		}

		return $this->maybe_override_field_value( $field_value, $form, $entry, $field_id );

	}

	/**
	 * Use the legacy gform_mailcamp_field_value filter instead of the framework gform_SLUG_field_value filter.
	 * @since  0.2
	 *
	 * @param string $field_value The field value.
	 * @param array  $form        The form object currently being processed.
	 * @param array  $entry       The entry object currently being processed.
	 * @param string $field_id    The ID of the field being processed.
	 *
	 * @return string
	 */
	public function maybe_override_field_value( $field_value, $form, $entry, $field_id ) {
		return gf_apply_filters( 'gform_mailcamp_field_value', array( $form['id'], $field_id ), $field_value, $form['id'], $field_id, $entry, $this->merge_var_name );
	}
	
	
	// # FIELD SETTINGS -------------------------------------------------------------------------------------------------

	
	
	// # FEED SETTINGS -------------------------------------------------------------------------------------------------

	/**
	 * Configures the settings which should be rendered on the feed edit page.
	 *
	 * @since  3.0
	 * @access public
	 *
	 * @return array
	 */
	public function feed_settings_fields() {

		$settings = array(
			
			/******************************
				Feed fields settings
			******************************/
			
			array(
				'title'  => esc_html__( 'Mailcamp Feed Settings', $this->_languageslug ),
				'fields' => array(
					array(
						'name'     => 'feedName',
						'label'    => esc_html__( 'Name', $this->_languageslug ),
						'type'     => 'text',
						'required' => true,
						'class'    => 'medium',
						'tooltip'  => sprintf(
							'<h6>%s</h6>%s',
							esc_html__( 'Name', 'gravityformsmailcamp' ),
							esc_html__( 'Enter a feed name to uniquely identify this setup.', $this->_languageslug )
						),
					),
					array(
						'name'     => 'mailcampList',
						'label'    => esc_html__( 'Mailcamp Mailinglist', $this->_languageslug ),
						'type'     => 'mailcamp_list',
						'required' => true,
						'tooltip'  => sprintf(
							'<h6>%s</h6>%s',
							esc_html__( 'Mailcamp mailinglist', $this->_languageslug ),
							esc_html__( 'Select the Mailchimp audience you would like to add your contacts to.', $this->_languageslug )
						),
					),
				),
			),
			
			
			/******************************
				Contact fields settings
			******************************/
			array(
				'title'  => esc_html__( 'Mailcamp Contact Settings', $this->_languageslug ),
				'fields' => array(
					array(
						'name'     => 'contact__confirmStatus',
						'label'    => esc_html__( 'Confirm status', $this->_languageslug ),
						'type'     => 'select',
						'required' => true,
						'tooltip'  => sprintf(
							'<h6>%s</h6>%s',
							esc_html__( 'Confirm status', $this->_languageslug ),
							esc_html__( 'The confirmation option is mostly used for the double opt-in process where contacts confirm a subscription by clicking on a link in a confirmation email. If you select unconfirmed you can send contacts an email with a confirmation link at a later time to make sure they want to be on the mailing list.', $this->_languageslug )
						),
					 	'choices'  => array(
						   	array(
							   	'label' => esc_html__( 'Confirmed', $this->_languageslug ),
							   	'value' => 'yes'
						   	),
						   	array(
							   	'label' => esc_html__( 'Not confirmed', $this->_languageslug ),
							   	'value' => 'no'
						   	)
						)
					),
					array(
						'name'     => 'contact__autoresponders',
						'label'    => esc_html__( 'Autoresponders', $this->_languageslug ),
						'type'     => 'select',
						'required' => true,
						'tooltip'  => sprintf(
							'<h6>%s</h6>%s',
							esc_html__( 'Autoresponders', $this->_languageslug ),
							esc_html__( 'If autoresponders have been created for this mailing list, these autoresponders will be sent if you check this option.', $this->_languageslug )
						),
					 	'choices'  => array(
						   	array(
							   	'label' => esc_html__( 'No', $this->_languageslug ),
							   	'value' => 'false'
						   	),
						   	array(
							   	'label' => esc_html__( 'Yes, add contacts to autoresponders', $this->_languageslug ),
							   	'value' => 'true'
						   	)
						)
					),
				),
			),
			
			/******************************
				Mapping fields
			******************************/
			array(
				'dependency' => 'mailcampList',
				'fields'     => array(
					array(
						'name'      => 'mappedFields',
						'label'     => esc_html__( 'Map Fields', $this->_languageslug ),
						'type'      => 'field_map',
						'field_map' => $this->merge_vars_field_map(),
						'tooltip'   => sprintf(
							'<h6>%s</h6>%s',
							esc_html__( 'Map Fields', $this->_languageslug ),
							esc_html__( 'Associate your MailCamp merge tags to the appropriate Gravity Form fields by selecting the appropriate form field from the list.', $this->_languageslug )
						),
					),
					/*
					array(
						'name'       => 'interestCategories',
						'label'      => esc_html__( 'Groups', $this->_languageslug ),
						'dependency' => array( $this, 'has_interest_categories' ),
						'type'       => 'interest_categories',
						'tooltip'    => sprintf(
							'<h6>%s</h6>%s',
							esc_html__( 'Groups', $this->_languageslug ),
							esc_html__( 'When one or more groups are enabled, users will be assigned to the groups in addition to being subscribed to the MailCamp audience. When disabled, users will not be assigned to groups.', $this->_languageslug )
						),
					),
					array(
						'name'    => 'options',
						'label'   => esc_html__( 'Options', 'gravityformsmailchimp' ),
						'type'    => 'checkbox',
						'choices' => array(
							array(
								'name'          => 'double_optin',
								'label'         => esc_html__( 'Double Opt-In', 'gravityformsmailchimp' ),
								'default_value' => 1,
								'onclick'       => 'if(this.checked){jQuery("#mailchimp_doubleoptin_warning").hide();} else{jQuery("#mailchimp_doubleoptin_warning").show();}',
								'tooltip'       => sprintf(
									'<h6>%s</h6>%s',
									esc_html__( 'Double Opt-In', 'gravityformsmailchimp' ),
									esc_html__( 'When the double opt-in option is enabled, Mailchimp will send a confirmation email to the user and will only add them to your Mailchimp audience upon confirmation.', 'gravityformsmailchimp' )
								),
							),
							array(
								'name'  => 'markAsVIP',
								'label' => esc_html__( 'Mark subscriber as VIP', 'gravityformsmailchimp' ),
							),
						),
					),
					array(
						'name'    => 'tags',
						'type'    => 'text',
						'class'   => 'medium merge-tag-support mt-position-right mt-hide_all_fields',
						'label'   => esc_html__( 'Tags', 'gravityformsmailchimp' ),
						'tooltip' => sprintf(
							'<h6>%s</h6>%s',
							esc_html__( 'Tags', 'gravityformsmailchimp' ),
							esc_html__( 'Associate tags to your Mailchimp contacts with a comma separated list (e.g. new lead, Gravity Forms, web source). Commas within a merge tag value will be created as a single tag.', 'gravityformsmailchimp' )
						),
					),
					array(
						'name'  => 'note',
						'type'  => 'textarea',
						'class' => 'medium merge-tag-support mt-position-right mt-hide_all_fields',
						'label' => esc_html__( 'Note', 'gravityformsmailchimp' ),
					),
					array(
						'name'    => 'optinCondition',
						'label'   => esc_html__( 'Conditional Logic', 'gravityformsmailchimp' ),
						'type'    => 'feed_condition',
						'tooltip' => sprintf(
							'<h6>%s</h6>%s',
							esc_html__( 'Conditional Logic', 'gravityformsmailchimp' ),
							esc_html__( 'When conditional logic is enabled, form submissions will only be exported to Mailchimp when the conditions are met. When disabled all form submissions will be exported.', 'gravityformsmailchimp' )
						),
					),
					*/
					array( 'type' => 'save' ),
				),
			),
		);
		
		// Get currently selected list/audience.
		$list = $this->get_setting( 'mailcampList' );
			
		return $settings;
	}
	

	/**
	 * Return an array of MailCamp list/audience fields which can be mapped to the Form fields/entry meta.
	 * @since  0.2
	 *
	 * @return array
	 */
	public function merge_vars_field_map() {

		// Initialize field map array.
		$field_map = array(
			'EMAIL' => array(
				'name'       => 'EMAIL',
				'label'      => esc_html__( 'Email Address', $this->_languageslug ),
				'required'   => true,
				'field_type' => array( 'email', 'hidden' ),
			),
		);

		// If unable to initialize API, return field map.
		if ( ! $this->initialize_api() ) {
			return $field_map;
		}

		// Get current list/audience ID.
		$list_id = $this->get_setting( 'mailcampList' );

		// Get merge fields.
		$merge_fields = $this->get_list_merge_fields( $list_id );
//		pr($merge_fields);

		if(!empty( $merge_fields)):
			foreach ( $merge_fields as $merge_field ):
//				pr($merge_field);
				// Define required field type.
				$field_type = null;

				// If this is an email merge field, set field types to "email" or "hidden".
				if ( 'EMAIL' === strtoupper( $merge_field['fieldtype'] ) ) {
					$field_type = array( 'email', 'hidden' );
				}

				// If this is an address merge field, set field type to "address".
				if ( 'address' === $merge_field['fieldtype'] ) {
					$field_type = array( 'address' );
				}

				// Add to field map.
				$field_map[ $merge_field['field_id'] ] = array(
					'name'       => $merge_field['field_id'],
					'label'      => $merge_field['name'],
					'required'   => $merge_field['required'],
					'field_type' => $field_type,
				);

			endforeach;
		endif;

		return $field_map;
	}
	
	
	/**
	 * Define the markup for the mailcamp_list type field
	 * @since  0.1
	 * @access public
	 *
	 * @param array $field The field properties.
	 * @param bool  $echo  Should the setting markup be echoed. Defaults to true.
	 *
	 * @return string
	 */
	public function settings_mailcamp_list( $field, $echo = true ) {
		// Initialize HTML string.
		$html = '';

		// If API is not initialized, return.
		if ( ! $this->initialize_api() ) {
			return $html;
		}
		
		// Prepare list/audience request parameters.
		$params = array();
		
		$lists = $this->api->get_lists( $params );
		
		
		// Initialize select options.
		$options = array(
			array(
				'label' => esc_html__( 'Select a Mailcamp Mailinglist', $this->_languageslug ),
				'value' => '',
			),
		);
		

		// Loop through Mailcamp lists/audiences.
		foreach ( $lists as $list ) {

			// Add list/audience to select options.
			$options[] = array(
				'label' => esc_html( $list['name'] ),
				'value' => esc_attr( $list['list_id'] ),
			);
		}
		
		
		// Add select field properties.
		$field['type']     = 'select';
		$field['choices']  = $options;
		$field['onchange'] = 'jQuery(this).parents("form").submit();';

		// Generate select field.
		$html = $this->settings_select( $field, false );
		return $html;
	}
	

	/**
	 * Get MailCamp merge fields for list/audience.
	 * @since  0.2
	 *
	 * @param string $list_id List/Audience ID to get merge fields for.
	 *
	 * @return array
	 */
	public function get_list_merge_fields( $list_id = '' ) {

		// If no list/audience ID was provided or if API cannot be initialized, return.
		if ( rgblank( $list_id ) || ! $this->initialize_api() ) {
			return array();
		}

		// If merge fields have already been retrieved, return.
		if ( isset( $this->merge_fields[ $list_id ] ) ) {
			return $this->merge_fields[ $list_id ];
		}

		try {
			// Get merge fields.
			$this->merge_fields[ $list_id ] = $this->api->get_list_merge_fields( ['list_id' => $list_id] );
		} catch ( Exception $e ) {
			// Log error.
			$this->log_error( __METHOD__ . '(): Unable to get merge fields for MailCamp audience; ' . $e->getMessage() );
			$this->merge_fields[ $list_id ] = array();
		}

		return $this->merge_fields[ $list_id ];

	}
	

	/**
	 * Get specific MailCamp merge field by tag.
	 * @since  0.2
	 *
	 * @param string $list_id List/Audience ID to get merge fields for.
	 * @param string $tag     Merge field tag.
	 *
	 * @return array
	 */
	public function get_list_merge_field( $list_id = '', $field_id = '' ) {

		// Get the merge fields for list/audience.
		$merge_fields = $this->get_list_merge_fields( $list_id );

		// If no merge fields were provided, return.
		if ( empty( $merge_fields ) || ! isset( $merge_fields['merge_fields'] ) ) {
			return;
		}

		// Loop through merge fields.
		foreach ( $merge_fields['merge_fields'] as $merge_field ) {

			// If this is not the merge field we are looking for, skip.
			if ( $field_id !== $merge_field['field_id'] ) {
				continue;
			}

			return $merge_field;

		}
		return array();
	}
	
	
	public function initialize_api() {

		// If API is already initialized, return true.
		if ( ! is_null( $this->api ) ) {
			return true;
		}
		
		$this->api = new GF_MailCamp_API();
		return true;
	}
}
